*****************************************************
ml2.c:163:void getusertree(FILE *itfp, cvector tr, int maxlen, int usebranch)
ml2.c:195:                      if (!usebranch) {
ml2.c:205:                      } else { /* if usebranch */


/* read user tree, drop all blanks, tabs, and newlines.
   Drop edgelengths (after :) but keep internal
   labels. Check whether all pairs of brackets match. */
void getusertree(FILE *itfp, cvector tr, int maxlen, int usebranch)
{
	int n, brac;
        char ci;

	/* look for opening bracket */
	n = 0;
	brac = 0;
	do {
		ci = fgetc(itfp);
		if (ci == EOF) {
			FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (missing start bracket in tree)\n\n\n");
			exit(1);
		}
		if (ci == '[') fremovecomment(itfp, &ci);
	} while ((ci != '('));
	tr[n] = ci;
	brac++;
	
	do {
		/* get next character (skip blanks, newlines, and tabs) */
		do {
			ci = fgetc(itfp);
			if (ci == EOF) {
				FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (no more characters in tree)\n\n\n");
				exit(1);
			}
			if (ci == '[') fremovecomment(itfp, &ci);
		} while (isspace((int) ci));
	
		/* remove edgelengths, if not needed (HAS) */
		if (ci == ':') { /* skip characters until a ,) appears  */
			if (!usebranch) {
				do {
					ci = fgetc(itfp);
					if (ci == EOF) {
						FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (missing ';' or ',' in tree)\n\n\n");
						exit(1);
					}
					if (ci == '[') fremovecomment(itfp, &ci);
				} while ((ci != ',' && ci != ')') );

			} else { /* if usebranch */
				n++;
				tr[n] = ci; /* add ':' to string */
				do {
					ci = fgetc(itfp);
					if (ci == EOF) {
						FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (missing ';' or ',' in tree)\n\n\n");
						exit(1);
					}
					if (ci == '[') fremovecomment(itfp, &ci);
					if(!isspace((int) ci) && (ci != ',') && (ci != ')')) {
						n++;
						tr[n] = ci;
					};
				} while ((ci != ',' && ci != ')') );
			}
		}
		
		if (ci == '(') {
			brac++;
		}
		if (ci == ')') {
			brac--;
		}

		n++;
		tr[n] = ci;
	
	} while ((ci != ';') && (n != maxlen-2));
	
	if (n == maxlen-2) {
		FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (tree description too long)\n\n\n");
		exit(1);
	}
	
	if (brac != 0) {
		FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (brackets don't match in tree)\n\n\n");
		FPRINTF(STDOUTFILE ">>%s<<\n\n\n", tr);
		exit(1);
	}
	
	n++;
	tr[n] = '\0';
} /* getusertree */


*****************************************************

Node *internalnode(Tree *tr, char **chpp, int *ninode)
{
	Node *xp;
	Node *np;
	Node *rp;
	int   i, j;		/* counter */
	int   dvg;		/* degree of subtree root node /degree vertex group ?*/
	int   ff;
	int   stop;
	int   numc;		/* char counter */
	char  ident[100];
	char  idcomp[11];
	char *idp;
	double blen;

	(*chpp)++;
	if (**chpp == '(') { /* process subgroup */
		
		xp = internalnode(tr, chpp, ninode);
		xp->isop = xp;
		dvg = 1;
		while (**chpp != ')') {
			if (**chpp == '\0') {
				FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (unexpected end of tree)\n\n\n");
				exit(1);
			}
			dvg++;
			/* insert edges around node */
			np = internalnode(tr, chpp, ninode);
			np->isop = xp->isop;
			xp->isop = np; 
			xp = np;
		}
		/* closing bracket reached */
		/* read edgelength !!!! (HAS) */
		
		(*chpp)++;
		if (dvg < 2) {
			FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (only one OTU inside pair of brackets)\n\n\n");
			exit(1);
		}
		
		if ((*ninode) >= Maxspc-3) { /* all internal nodes already used */
			FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (no unrooted tree)\n\n\n");
			exit(1);
		}

		rp = tr->ibrnchp[*ninode];
		rp->isop = xp->isop;
		xp->isop = rp;

		for (j = 0; j < Numspc; j++)
			rp->paths[j] = 0;
		xp = rp->isop;
		while (xp != rp) {
			for (j = 0; j < Numspc; j++) {
				if (xp->paths[j] == 1)
					rp->paths[j] = 1;
			}
			xp = xp->isop;
		}
		(*ninode)++;

		if ((**chpp) == ',' || (**chpp) == ')') return rp->kinp;
		if ((**chpp) == '\0')  {
			FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (unexpected end of tree)\n\n\n");
			exit(1);
		}

		if ((**chpp) != ':') {

			/* read internal label into rp->label (max. 20 characters) */
			rp->label = new_cvector(21);
/*
			(rp->label)[0] = **chpp;
*/
			(rp->label)[0] = '\0';
			for (numc = 0; ((numc < 20) && ((**chpp) != ':')); numc++) {
				if ((**chpp) == ',' || (**chpp) == ')') return rp->kinp;
				if ((**chpp) == '\0')  {
					FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (unexpected end of tree)\n\n\n");
					exit(1);
				}
				(rp->label)[numc] = **chpp;
				(rp->label)[numc+1] = '\0';
				(*chpp)++;
			}	

			/* skip the rest of the internal label */
			while (((**chpp) != ',') && ((**chpp) != ')') && ((**chpp) != ':')) {
				(*chpp)++;
				if ((**chpp) == '\0')  {
					FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (unexpected end of tree)\n\n\n");
					exit(1);
				}
			}
		} /* if ! ':' */

		/* read edgelength !!!! (HAS) */
		if (**chpp == ':') {
			(void) sreadedgelength(&blen, chpp);
			rp->lengthext=blen;
			rp->lengthset=1;
			rp->kinp->lengthext=blen;
			rp->kinp->lengthset=1;
		}

			
		return rp->kinp;
		
	} else { /* process species names */
		/* read species name */
		for (idp = ident; 
		     **chpp != ',' && **chpp != ')' && **chpp != ':' && **chpp != '\0'; 
		     (*chpp)++) {
			*idp++ = **chpp;	
		}
		*idp = '\0';

		/* look for internal number */
		idcomp[10] = '\0';
		
		for (i = 0; i < Maxspc; i++) {
			ff = 0;
			stop = FALSE;
			do {
				idcomp[ff] = Identif[i][ff];
				ff++;
				if (idcomp[ff-1] == ' ') stop = TRUE;
			} while (!stop && (ff != 10));
			if (stop) idcomp[ff-1] = '\0';
			
			if (!strcmp(ident, idcomp)) {
				if (usedtaxa[i]) {
					FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (multiple occurence of sequence '");
					FPRINTF(STDOUTFILE "%s' in tree)\n\n\n", ident);
					exit(1);
				}
				usedtaxa[i] = TRUE;

				/* read edgelength !!!! (HAS) */
				if (**chpp == ':') {
					(void) sreadedgelength(&blen, chpp);
					tr->ebrnchp[i]->lengthext=blen;
					tr->ebrnchp[i]->lengthset=1;
					tr->ebrnchp[i]->kinp->lengthext=blen;
					tr->ebrnchp[i]->kinp->lengthset=1;
				}

				return tr->ebrnchp[i]->kinp;
			}
		}
		
		FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (unknown sequence '%s' in tree)\n\n\n", ident);
		exit(1);
	}
	return NULL; /* never returned but without some compilers complain */
} /* internalnode */


*****************************************************
ml2.c:415:void constructtree(Tree *tr, cvector strtree, int usebranch)


/* make tree structure, the tree description may contain internal
    labels but no edge lengths */
void constructtree(Tree *tr, cvector strtree, int usebranch)
{
	char *chp;		/* char ptr to tree string */
	int   ninode;		/* number of internal nodes < Maxspc-3*/
	int   i;		/* counter */
	int   dvg;		/* degree vertex ? */
	int   numc;		/* char counter */
	Node *xp;		/* last read Node/Subtree */
	Node *np;		/* newly read Node/Subtree */

	ninode = 0;
	chp = strtree;
	usedtaxa = new_ivector(Maxspc);
	for (i = 0; i < Maxspc; i++) usedtaxa[i] = FALSE;

	xp = internalnode(tr, &chp, &ninode);
	xp->isop = xp;
	dvg = 1;
	while (*chp != ')') { /* look for closing bracket */
		if (*chp == '\0') {
			FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (unexpected end of tree)\n\n\n");
			exit(1);
		}
		dvg++;
		/* insert edges around node */
		np = internalnode(tr, &chp, &ninode);
		np->isop = xp->isop;
		xp->isop = np; 
		xp = np;
	}
	
	for (i = 0; i < Maxspc; i++)
		if (usedtaxa[i] == FALSE) {
			FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (sequences missing in tree)\n\n\n");
			exit(1);	
		}
	
	/* closing bracket reached */
	if (dvg < 3) {
		FPRINTF(STDOUTFILE "\n\n\nUnable to proceed (no unrooted tree)\n\n\n");
		exit(1);
	}
	tr->rootp = xp;
	Numibrnch = ninode;
	Numbrnch = Numspc + ninode;

	chp++;
	if (*chp == ';' || *chp == '\0') {
		free_ivector(usedtaxa);
		return;
	}

	/* copy last internal label (max. 20 characters) */
	xp->label = new_cvector(21);
	(xp->label)[0] = *chp;
	(xp->label)[1] = '\0';		
	for (numc = 1; numc < 20; numc++) {
		chp++;
		if (*chp == ';' || *chp == '\0') {
			free_ivector(usedtaxa);
			return;
		} else {
			(xp->label)[numc] = *chp;
			(xp->label)[numc+1] = '\0';
		}
	}
	free_ivector(usedtaxa);	
	return;
} /* constructtree */




*****************************************************
ml2.c:544:void makeusertree(FILE *itfp, int usebranchlen)
ml2.c:549:      getusertree(itfp, strtree, 53*Maxspc, usebranchlen);
ml2.c:551:      constructtree(Ctree, strtree, usebranchlen);

void makeusertree(FILE *itfp, int usebranchlen)
{
	cvector strtree;
	
	strtree = new_cvector(53*Maxspc); /* for treefile */ 
	getusertree(itfp, strtree, 53*Maxspc, usebranchlen);
	removebasalbif(strtree);
	constructtree(Ctree, strtree, usebranchlen);
	free_cvector(strtree);
}

*****************************************************
ml2.c:904:void readusertree(FILE *ifp, int usebranch)
ml2.c:918:      makeusertree(ifp, usebranch);


/* read usertree from file to memory */
void readusertree(FILE *ifp, int usebranch)
{
	/* reserve memory for tree if necessary */
	if (mlmode != 2) { /* no tree */
		if (Ctree != NULL)
			free_tree(Ctree, Numspc);
		Ctree = new_tree(Maxspc, Numptrn, Seqpat);
		Numbrnch = 2*Maxspc-3;
		Numibrnch = Maxspc-3;
		Numspc = Maxspc;
		mlmode = 2;
	}

	/* read tree */
	makeusertree(ifp, usebranch);
} /* readusertree */

*****************************************************
ml2.c:923:double usertree_lklhd(int usebranch)
ml2.c:930:      if(usebranch) {


/* compute the likelihood of a usertree */
double usertree_lklhd(int usebranch)
{
	/* be sure to have a usertree in memory and
	   to have pairwise distances computed */

	clockmode = 0; /* nonclocklike branch lengths */

	if(usebranch) {
		/* set preset barnch lengths from usertree file */
		setextlength(Ctree, Numspc, Numibrnch, Brnlength);

		/* compute likelihood */
		Ctree->lklhd = treelkl(Ctree);

	} else {
		/* least square estimate for branch length */
		changedistan(Distanmat, Distanvec, Numspc);	
		lslength(Ctree, Distanvec, Numspc, Numibrnch, Brnlength);

		/* compute and optimize likelihood */
		Ctree->lklhd = optlkl(Ctree);
	}

	return Ctree->lklhd;
} /* usertree_lklhd */


*****************************************************
ml2.c:952:double usertree_alklhd(int usebranch)
ml2.c:959:      if(usebranch) {

/* compute the approximate likelihood of a usertree */
double usertree_alklhd(int usebranch)
{
	/* be sure to have a usertree in memory and
	   to have pairwise distances computed */

	clockmode = 0; /* nonclocklike branch lengths */

	if(usebranch) {
		/* set preset barnch lengths from usertree file */
		setextlength(Ctree, Numspc, Numibrnch, Brnlength);

	} else {
		/* least square estimate for branch length */
		changedistan(Distanmat, Distanvec, Numspc);
		lslength(Ctree, Distanvec, Numspc, Numibrnch, Brnlength);
	}



	/* compute likelihood */
	Ctree->lklhd = treelkl(Ctree);

	return Ctree->lklhd;
} /* usertree_alklhd */


