
#define EXTERN 
#define TRUE 1
#define FALSE 0
#define HEPSILON   0.0001  /* error in node and root heights */    


/* prototypes */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <ctype.h>
/* #include "util.h" */
/* #include "gamma.h" */

/***double onedimenmin(double, double, double, double (*f )(double ), double,
 double *, double *);***/
typedef double *dvector, **dmatrix, ***dcube;
typedef char *cvector, **cmatrix, ***ccube;
typedef int *ivector;


/* variables */
cmatrix Idseq1;            /* identifiers of seqs in alldatafile 1 */
cmatrix aseq1;             /* seqs of alldatafile 1 */
int Maxspcone;             /* number of seqs in alldatafile 1 */
int Maxind;                /* number of individuals in infofile */
int MaxseqL;
int site;
ivector Pairs;
double all;
double ALPHA;
dmatrix distanceMatrix; 
dvector Timevec;
FILE *aseq1fp;
FILE *matfp;
FILE *dzout;

/* prototypes */
int fputidone(FILE *ofp, int t, cmatrix Identif);   /* print identifier of seqs in alldatafile 1 */
void fputidone10(FILE *ofp, int t, cmatrix Identif);/* print identifier of seqs in alldatafile 1 in 10 char lengh */
int fputaseq1(FILE *ofp, int t);                    /*print sequences of alldatafile 1*/
void loadpairs(int pos);
void computetimes();
double funk(double alph);


/*
 * memory allocation error handler
 */

void maerror(char *message)
{
  printf("\n\n\nUnable to proceed (lack of memory: %s)\n\n", message);
  exit(1);
}

dmatrix new_dmatrix(int nrow, int ncol)
{
  int i;
  dmatrix m;

  m = (dmatrix) malloc((unsigned) (nrow * sizeof(dvector)));
  if (m == NULL) maerror("step 1 in in new_dmatrix");

  *m = (dvector) malloc((unsigned) (nrow * ncol * sizeof(double)));
  if (*m == NULL) maerror("step 2 in in new_dmatrix");

  for (i = 1; i < nrow; i++) m[i] = m[i-1] + ncol;

  return m;
}

cmatrix new_cmatrix(int nrow, int ncol)
{
  int i;
  cmatrix m;

  m = (cmatrix) malloc((unsigned) (nrow * sizeof(cvector)));
  if (m == NULL) maerror("step 1 in new_cmatrix");

  *m = (cvector) malloc((unsigned) (nrow * ncol * sizeof(char)));
  if (*m == NULL) maerror("step 2 in new_cmatrix");

  for (i = 1; i < nrow; i++) m[i] = m[i-1] + ncol;

  return m;
}

void free_cmatrix(cmatrix m)
{
  free((char *) *m);
  free((char *) m);
}

/*****************************************************************************/
/* minimization of a function by Brents method (Numerical Recipes)           */
/*****************************************************************************/

#define ITMAX 100
#define CGOLD 0.3819660
#define GOLD 1.618034
#define GLIMIT 100.0
#define TINY 1.0e-20
#define ZEPS 1.0e-10
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

/* Brents method in one dimension */
double brent(double ax, double bx, double cx, double (*f)(double), double tol,
	     double *foptx, double *f2optx, double fax, double fbx, double fcx)
{
  int iter;
  double a,b,d=0,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
  double xw,wv,vx;
  double e=0.0;

  a=(ax < cx ? ax : cx);
  b=(ax > cx ? ax : cx);
  x=bx;
  fx=fbx;
  if (fax < fcx) {
    w=ax;
    fw=fax;
    v=cx;
    fv=fcx;
  } else {
    w=cx;
    fw=fcx;
    v=ax;
    fv=fax;	
  }
  for (iter=1;iter<=ITMAX;iter++) {
    xm=0.5*(a+b);
    tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
    if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
      *foptx = fx;
      xw = x-w;
      wv = w-v;
      vx = v-x;
      *f2optx = 2.0*(fv*xw + fx*wv + fw*vx)/
	(v*v*xw + x*x*wv + w*w*vx);
      return x;
    }
    if (fabs(e) > tol1) {
      r=(x-w)*(fx-fv);
      q=(x-v)*(fx-fw);
      p=(x-v)*q-(x-w)*r;
      q=2.0*(q-r);
      if (q > 0.0) p = -p;
      q=fabs(q);
      etemp=e;
      e=d;
      if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
	d=CGOLD*(e=(x >= xm ? a-x : b-x));
      else {
	d=p/q;
	u=x+d;
	if (u-a < tol2 || b-u < tol2)
	  d=SIGN(tol1,xm-x);
      }
    } else {
      d=CGOLD*(e=(x >= xm ? a-x : b-x));
    }
    u=(fabs(d) >= tol1 ? x+d : x+SIGN(tol1,d));
    fu=(*f)(u);
    if (fu <= fx) {
      if (u >= x) a=x; else b=x;
      SHFT(v,w,x,u)
	SHFT(fv,fw,fx,fu)
	} else {
	  if (u < x) a=u; else b=u;
	  if (fu <= fw || w == x) {
	    v=w;
	    w=u;
	    fv=fw;
	    fw=fu;
	  } else if (fu <= fv || v == x || v == w) {
	    v=u;
	    fv=fu;
	  }
	}
  }
  *foptx = fx;
  xw = x-w;
  wv = w-v;
  vx = v-x;
  *f2optx = 2.0*(fv*xw + fx*wv + fw*vx)/
    (v*v*xw + x*x*wv + w*w*vx);
  return x;
}


/*
  one-dimensional minimization - as input a lower and an upper limit and a 
  trial value for the minimum is needed: xmin < xguess < xmax
  the function and a fractional tolerance has to be specified
  onedimenmin returns the optimal x value and the value of the function
  and its second derivative at this point
*/
double onedimenmin(double xmin, double xguess, double xmax, 
		   double (*f)(double), double tol, double *fx, double *f2x)
{
  double eps, optx, ax, bx, cx, fa, fb, fc;
		
  /* first attempt to bracketize minimum */
  eps = xguess*tol*50.0;
  ax = xguess - eps;
  if (ax < xmin) ax = xmin;
  bx = xguess;
  cx = xguess + eps;
  if (cx > xmax) cx = xmax;
  
  /* check if this works */
  fa = (*f)(ax);
  fb = (*f)(bx);
  fc = (*f)(cx);

  /* if it works use these borders else be conservative */
  if ((fa < fb) || (fc < fb)) {
    if (ax != xmin) fa = (*f)(xmin);
    if (cx != xmax) fc = (*f)(xmax);
    optx = brent(xmin, xguess, xmax, f, tol, fx, f2x, fa, fb, fc);
  } else
    optx = brent(ax, bx, cx, f, tol, fx, f2x, fa, fb, fc);

  return optx; /* return optimal x */
}


/*****************************************************************************/
/* sequence in,output                                                        */
/*****************************************************************************/


/******************* sequences of alldatafile 1 ******************************/

/* read ten characters of current line as identifier */
void readidone(FILE *infp, int t)
{
  int i, flag;

  for (i = 0; i < 10; i++) {
    Idseq1[t][i] = fgetc(infp);
    if (ferror(infp) || !isprint(Idseq1[t][i]) ) {
      fprintf(stderr, "\nERROR: Missing full sequence name");
      fprintf(stderr, " (sequence # %d)!\n", t + 1);
      exit(1);
    }
  }

	
/* convert leading blanks in sequence name to underscores */
  flag = FALSE;
  for (i = 9; i > -1; i--) {
    if (flag == FALSE) {
      if (Idseq1[t][i] != ' ') flag = TRUE; 
    } else {
      if (Idseq1[t][i] == ' ') Idseq1[t][i] = '_';
    }
  }
}

/* read next allowed character */
char readnextcharacterone(FILE *ifp, int ind, int nsite)
{
  char c;

  /* ignore blanks and control characters except newline */
  do {
    if (fscanf(ifp, "%c", &c) != 1) {
      fprintf(stderr, "\nERROR: Missing character");
      fprintf(stderr, " (sequence # %d [", ind + 1);
      fputidone(stderr, ind, Idseq1);
      fprintf(stderr, "], ");
      fprintf(stderr, "character # %d)!\n", nsite + 1);
      exit(1);
    }
  } while (c == ' ' || (iscntrl(c) && c != '\n'));
  return c;
}

/* skip rest of the line */
void skiprestoflineone(FILE* ifp, int ind, int nsite)
{
  char c;

  /* read chars until the first newline */
  do{
    c = fgetc(ifp);
    if (ferror(ifp)) {
      fprintf(stderr, "\nERROR: Missing new line");
      fprintf(stderr, " (sequence # %d [", ind + 1);
      fputidone(stderr, ind, Idseq1);
      fprintf(stderr, "], character # %d)!\n", nsite + 1);
      exit(1);
    }
  } while (c != '\n');
}

/* skip control characters and blanks */
void skipcntrlone(FILE *ifp, int ind, int nsite)
{
  char c;

  /* read over all control characters and blanks */
  do {
    c = fgetc(ifp);
    if (ferror(ifp)) {
      fprintf(stderr, "\nERROR: Missing character ");
      fprintf(stderr, " (sequence # %d [", ind + 1);
      fputidone(stderr, ind, Idseq1);
      fprintf(stderr, "], character # %d)!\n", nsite + 1);
      exit(1);
    }
  } while (iscntrl(c) || c == ' ');
  /* go one character back */
  if (fseek(ifp, -1, SEEK_CUR)) {
    fprintf(stderr, "\nERROR: File positioning error");
    fprintf(stderr, " (sequence # %d [", ind + 1);
    fputidone(stderr, ind, Idseq1);
    fprintf(stderr, "], character # %d)!\n", nsite + 1);
    exit(1);
  }
}



/* read sequences of one data set */
void getseqsone(FILE *ifp)
{
  int spc, nsite, endofline, linelength, i;
  char c;
  
  aseq1 = new_cmatrix(Maxspcone, MaxseqL);

  /* read all characters */
  nsite = 0; /* next site to be read */

  while (nsite < MaxseqL) {

    /* read first sequence */
    spc = 0;

    /* go to next true line */
    skiprestoflineone(ifp, spc, nsite);
    skipcntrlone(ifp, spc, nsite);

    if (nsite == 0) 
      readidone(ifp, spc);

    endofline = FALSE;
    linelength = 0;		

    do {
      c = readnextcharacterone(ifp, spc, nsite + linelength);
      if (c == '\n') 
	endofline = TRUE;
      else if (c == '.') {

	fprintf(stderr,
		"\nERROR: Character '.' not valid in first sequence");
	fprintf(stderr, " ([");
	fputidone(stderr, spc, Idseq1);
	fprintf(stderr, "], character # %d)!\n",
		nsite + linelength + 1);
	exit(1);

      } else if (nsite + linelength < MaxseqL) {

	/* change to upper case */
	aseq1[spc][nsite + linelength] = toupper(c);
	linelength++;

      } else {

	endofline = TRUE;
	skiprestoflineone(ifp, spc, nsite + linelength);

      }
    } while (!endofline);	

    if (linelength == 0) {
      fprintf(stderr, "\nERROR: Line with length zero ");
      fprintf(stderr, " (sequence # %d [", spc + 1);
      fputidone(stderr, spc, Idseq1);
      fprintf(stderr, "], character # %d)!\n", nsite + 1);
      exit(1);
    }

    /* read other taxa */
    for (spc = 1; spc < Maxspcone; spc++) {
      /* go to next true line */
      if (spc != 1) 
	skiprestoflineone(ifp, spc, nsite);
      skipcntrlone(ifp, spc, nsite);
      if (nsite == 0) readidone(ifp, spc);
      for (i = nsite; i < nsite + linelength; i++) {
	c = readnextcharacterone(ifp, spc, i);
	if (c == '\n') { /* too short */
	  fprintf(stderr, "\nERROR: Line too short");
	  fprintf(stderr, " (sequence # %d [", spc + 1);
	  fputidone(stderr, spc, Idseq1);
	  fprintf(stderr, "], character # %d)!\n", i + 1);
	  exit(1);
	} else if (c == '.') {
	  aseq1[spc][i] = aseq1[0][i];
	} else {
	  /* change to upper case */
	  aseq1[spc][i] = toupper(c);
	}
      }
    }
    nsite = nsite + linelength;
  }
}



/* initialize seq identifer array */
void initidone(int t) {

  int i, j;
  
  Idseq1 = new_cmatrix(t, 10);

  for (i = 0; i < t; i++)
    for (j = 0; j < 10; j++)
      Idseq1[i][j] = ' ';

}


/* read the data sets */

void getseqsoneset(FILE *ifp) {      

  fscanf (aseq1fp, "%d %d", &Maxspcone, &MaxseqL);
  initidone (Maxspcone);
  getseqsone (ifp);

}



/* print identifier in full 10 char length */
void fputidone10(FILE *ofp, int t, cmatrix Identif) {
	
  int i;

  for (i = 0; i < 10; i++) 
    fputc(Identif[t][i], ofp);

}


/* print identifier of specified sequence up to first space */
int fputidone(FILE *ofp, int t, cmatrix Identif) {	

  int i;
	
  i = 0;

  while (Identif[t][i] != ' ' && i < 10) {

    fputc(Identif[t][i], ofp);
    i++;

  }

  return i;

}





/* print sequences of specified sequence up to first space */
int fputaseq1(FILE *ofp, int t) {	

  int i;
	
  i = 0;
  while (aseq1[t][i] != ' ' && i < MaxseqL) {

    fputc(aseq1[t][i], ofp);
    i++;

  }

  return i;

}

/*****************************************************************************/
/* open, close files                                                         */
/*****************************************************************************/

/* open input files */
void openinputfiles(char *infile, char *matfile)
{		

  /* open sequencefile  */
  if ((aseq1fp = fopen(infile, "r")) == NULL) {
    fprintf(stderr,
	    "\nERROR: Missing sequence input file named infile %s!\n",
	    infile);                   
  }  

  /* open distancefile  */
  if ((matfp = fopen(matfile, "r")) == NULL) {
    fprintf(stderr,
	    "\nERROR: Missing distancematrixfile named mat %s!\n",
	    matfile);
  }
   
}

/* close files */
void closefiles() {

  fclose(aseq1fp);
  fclose(matfp);
  fclose(dzout);

}


/*****************************************************************************/
/* routines                                                                  */
/*****************************************************************************/
void loadpairs(int pos){

  int k,l;

  fprintf(stderr,"\n eineinhalb"); 
                   
  for(k = 0; k<(Maxspcone/2); k++){

    l = k*2;

    if((aseq1[l][pos]) != (aseq1[l+1][pos]))
      Pairs[k] = 1;

  }

}


void computetimes(){ 

  double sumtime,time;
  int  l;

  sumtime = 0.0;
  time = 0.0;
 
  for ( l = Maxspcone; l > (Maxspcone/2); l--){
    time =(2.00*(2.00/(((double)l)*((double)(l-1))))); 
    sumtime += time;  
    /*** sumtime = ( 0.2 * 0.1250);***/               
    Timevec[(Maxspcone-l)]=sumtime;
  }                                       
 
}


double funk(double alph){

  double erg, icksalph,d1, help2;
  int k,l,diff;
                
  erg = 0.0;
  icksalph = 0.0;
  d1 = 0.0;         
  help2 = 0.0; 
                  
  for ( k =0; k < Maxspcone; k++)

    for ( l =k+1; l < Maxspcone; l++){
      diff = 0;
                                
      if((aseq1[k][site]) != (aseq1[l][site]))
	diff = 1;

      icksalph =  exp(((-alph)*(4.00/3.00)*(distanceMatrix[k][l])));
       
      if(diff == 0){

	d1 =  (0.25 + (0.75*icksalph ));
	erg += log(d1);  

      } else {          
                               
	d1 = (0.75 - (0.75 * icksalph));
	erg +=  log(d1);

      }
        	     
    }  

  return -erg;  
               
}

/*****************************************************************************/
/* main part                                                                 */
/*****************************************************************************/

int main(int argc, char *argv[])
{       
                                  
  int l,k;            
  double avrgalph, alpha;	
  double fx, f2x, minalph, maxalph, alphguess,d;                              
  double *alphvec;

  if (argc != 4) {

    fprintf (stderr, "pairratesAP Inputfile MATfile Outputfile\n");
    exit (1);

  }

  ALPHA = 1.0;
   	           
  fprintf(stderr,"\n %d, %d, %lf \n",Maxspcone, MaxseqL,ALPHA);   
                                                               
  /* get sequences and information */
  fprintf(stderr, "\nINFO:  Looking for sequence input data\n");
  openinputfiles(argv [1], argv [2]);
 
  getseqsoneset(aseq1fp);
 
  dzout = fopen(argv [3], "w");
  
  distanceMatrix = new_dmatrix(Maxspcone, Maxspcone); 
  for(k = 0; k<Maxspcone; k++ ){                
    for(l = 0; l<Maxspcone; l++ ){         
      distanceMatrix[k][l] = 0.0;      
    }     
  }      
  
  for(k = 0; k<Maxspcone; k++ ){              
    for(l = 0; l<Maxspcone; l++ ){ 
      fscanf(matfp,"%lf", &d);
      distanceMatrix[k][l] = d;
    }        	                                                    
  }
  
  alphguess = 50.00;
  avrgalph = 0.0;      

  alphvec = (double *) malloc (MaxseqL * sizeof (double));

  if (alphvec == NULL) {
    fprintf (stderr, "Out of memory!\n");
    exit (1);
  }

  for(site =0;site < MaxseqL; site++){  
    
    /*** Pairs ist 1, wenn paar  verschieden, Zeit bei index 0 ist t_sasi, bei 
	 index 1 t_sasi-1..usw. *******/ 
    
    fx = 0.0;
    f2x = 0.0;
    alpha = alphguess;
    minalph = 0.000000001;
    maxalph = 100.0;     

    /* optimization */

    alphvec [site] = onedimenmin(minalph, alpha, maxalph, funk, HEPSILON, &fx, &f2x);  
    
/*     fprintf(dzout, "%.4f   \n",alpha);   */
    
    avrgalph += alphvec [site];    
  } 
  
  
  avrgalph = (avrgalph/ MaxseqL);
  
/*   fprintf(stderr, "\nERG fuer %.2f: %.4f\n",alphguess,avrgalph);  */
  
/*   fprintf(dzout, "\nERG fuer %.2f: %.4f\n",alphguess,avrgalph);    */
  
  
  for (site = 0; site < MaxseqL; site++)
     fprintf(dzout, "%.4f   \n",alphvec [site] / avrgalph);  
   
  /* garbage collection */
  free_cmatrix(Idseq1);
  free_cmatrix(aseq1);
  
  /* close files */
  closefiles();
  
  return 0;
}
