#define EXTERN 
#define TRUE 1
#define FALSE 0
    
/* prototypes */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/times.h>

typedef char *cvector, **cmatrix;
typedef int *ivector;

/* variables */
cmatrix Idseq1;               /* identifiers of seqs in alldatafile 1 */
cmatrix aseq1;                /* seqs of alldatafile 1 */
int Maxspc;                   /* number of seqs in alldatafile 1 */
int MaxseqL;                  /* number of charakters per seq */
FILE *aseq1fp;
FILE *oseqfp;
FILE *dzout;
/* prototypes */
int fputidone(FILE *ofp, int t, cmatrix Identif); /* print identifier of seqs in alldatafile 1 */
void fputidone10(FILE *ofp, int t, cmatrix Identif); /* print identifier of seqs in alldatafile 1 in 10 char lengh */
int fputaseq1(FILE *ofp, int t, int MaxseqL); /*print sequences of alldatafile 1*/


/*  Random number generator from Numerical recipes (Ed 2, ran1) */

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double rand01(long *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  double temp;
  if(*idum<=0 || !iy) {
    if(-(*idum) < 1) *idum=1;
      else *idum= -(*idum);
    for(j=NTAB+7;j>=0;j--) {
        k=(*idum)/IQ;
        *idum=IA*(*idum-k*IQ) -IR*k;
        if(*idum < 0) *idum += IM;
        if(j <NTAB) iv[j]= *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if(*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j]= *idum;
  if((temp=AM*iy) > RNMX) return RNMX;
    else return temp;
}


void maerror(char *message)
{
  fprintf(stderr, "\nERROR: Memory allocation failure in %s!\n", message);
  exit(1);
}

cmatrix new_cmatrix(int nrow, int ncol)
{
  int i;
  cmatrix m;

  m = (cmatrix) malloc((unsigned) (nrow * sizeof(cvector)));
  if (m == NULL) maerror("cmatrix [step 1]");
	
  *m = (cvector) malloc((unsigned) (nrow * ncol * sizeof(char)));
  if (*m == NULL) maerror("cmatrix [step 2]");

  for (i = 1; i < nrow; i++) m[i] = m[i-1] + ncol;

  return m;
}

void free_cmatrix(cmatrix m)
{
  free((char *) *m);
  free((char *) m);
}


ivector new_ivector(int n)
{
  ivector v;

  v = (ivector) malloc((unsigned) (n * sizeof(int)));
  if (v == NULL) maerror("ivector");

  return v;
}


/*****************************************************************************/
/* sequence in,output                                                        */
/*****************************************************************************/



/* read ten characters of current line as identifier */
void readidone(FILE *infp, int t)
{
  int i, flag;
  
  for (i = 0; i < 10; i++) {
    Idseq1[t][i] = fgetc(infp);
    if (ferror(infp) || !isprint(Idseq1[t][i]) ) {
      fprintf(stderr, "\nERROR: Missing full sequence name");
      fprintf(stderr, " (sequence # %d)!\n", t + 1);
      exit(1);
    }
  }
  
  
  /* convert leading blanks in sequence name to underscores */
  flag = FALSE;
  for (i = 9; i > -1; i--) {
    if (flag == FALSE) {
      if (Idseq1[t][i] != ' ') flag = TRUE; 
    } else {
      if (Idseq1[t][i] == ' ') Idseq1[t][i] = '_';
    }
  }
}

/* read next allowed character */
char readnextcharacterone(FILE *ifp, int ind, int nsite)
{
  char c;
  
  /* ignore blanks and control characters except newline */
  do {
    if (fscanf(ifp, "%c", &c) != 1) {
      fprintf(stderr, "\nERROR: Missing character");
      fprintf(stderr, " (sequence # %d [", ind + 1);
      fputidone(stderr, ind, Idseq1);
      fprintf(stderr, "], ");
      fprintf(stderr, "character # %d)!\n", nsite + 1);
      exit(1);
    }
  } while (c == ' ' || (iscntrl(c) && c != '\n'));
  return c;
}

/* skip rest of the line */
void skiprestoflineone(FILE* ifp, int ind, int nsite)
{
  char c;
  
  /* read chars until the first newline */
  do{
    c = fgetc(ifp);
    if (ferror(ifp)) {
      fprintf(stderr, "\nERROR: Missing new line");
      fprintf(stderr, " (sequence # %d [", ind + 1);
      fputidone(stderr, ind, Idseq1);
      fprintf(stderr, "], character # %d)!\n", nsite + 1);
      exit(1);
    }
  } while (c != '\n');
}

/* skip control characters and blanks */
void skipcntrlone(FILE *ifp, int ind, int nsite)
{
  char c;
  
  /* read over all control characters and blanks */
  do {
    c = fgetc(ifp);
    if (ferror(ifp)) {
      fprintf(stderr, "\nERROR: Missing character ");
      fprintf(stderr, " (sequence # %d [", ind + 1);
      fputidone(stderr, ind, Idseq1);
      fprintf(stderr, "], character # %d)!\n", nsite + 1);
      exit(1);
    }
  } while (iscntrl(c) || c == ' ');
  /* go one character back */
  if (fseek(ifp, -1, SEEK_CUR)) {
    fprintf(stderr, "\nERROR: File positioning error");
    fprintf(stderr, " (sequence # %d [", ind + 1);
    fputidone(stderr, ind, Idseq1);
    fprintf(stderr, "], character # %d)!\n", nsite + 1);
    exit(1);
  }
}



/* read sequences of one data set */
void getseqsone(FILE *ifp)
{
  int spc, sn, co, nsite, endofline, linelength, i;
  char c;
  
  aseq1 = new_cmatrix(Maxspc, MaxseqL);
  /* read all characters */
  nsite = 0; /* next site to be read */
  while (nsite < MaxseqL) {
    /* read first sequence */
    spc = 0;
    /* go to next true line */
    skiprestoflineone(ifp, spc, nsite);
    skipcntrlone(ifp, spc, nsite);
    if (nsite == 0) readidone(ifp, spc);
    endofline = FALSE;
    linelength = 0;		
    do {
      c = readnextcharacterone(ifp, spc, nsite + linelength);
      if (c == '\n') endofline = TRUE;
      else if (c == '.') {
	fprintf(stderr,
		"\nERROR: Character '.' not valid in first sequence");
	fprintf(stderr, " ([");
	fputidone(stderr, spc, Idseq1);
	fprintf(stderr, "], character # %d)!\n",
		nsite + linelength + 1);
	exit(1);
      } else if (nsite + linelength < MaxseqL) {
	/* change to upper case */
	aseq1[spc][nsite + linelength] = toupper(c);
	linelength++;
      } else {
	endofline = TRUE;
	skiprestoflineone(ifp, spc, nsite + linelength);
      }
    } while (!endofline);	
    if (linelength == 0) {
      fprintf(stderr, "\nERROR: Line with length zero ");
      fprintf(stderr, " (sequence # %d [", spc + 1);
      fputidone(stderr, spc, Idseq1);
      fprintf(stderr, "], character # %d)!\n", nsite + 1);
      exit(1);
    }
    /* read other taxa */
    for (spc = 1; spc < Maxspc; spc++) {
      /* go to next true line */
      if (spc != 1) skiprestoflineone(ifp, spc, nsite);
      skipcntrlone(ifp, spc, nsite);
      if (nsite == 0) readidone(ifp, spc);
      for (i = nsite; i < nsite + linelength; i++) {
	c = readnextcharacterone(ifp, spc, i);
	if (c == '\n') { /* too short */
	  fprintf(stderr, "\nERROR: Line too short");
	  fprintf(stderr, " (sequence # %d [", spc + 1);
	  fputidone(stderr, spc, Idseq1);
	  fprintf(stderr, "], character # %d)!\n", i + 1);
	  exit(1);
	} else if (c == '.') {
	  aseq1[spc][i] = aseq1[0][i];
	} else {
	  /* change to upper case */
	  aseq1[spc][i] = toupper(c);
	}
      }
    }
    nsite = nsite + linelength;
  }
}



/* initialize seq identifer array */
void initidone(int t)
{
  int i, j;
  
  Idseq1 = new_cmatrix(t, 10);
  for (i = 0; i < t; i++)
    for (j = 0; j < 10; j++)
      Idseq1[i][j] = ' ';
}




/* read the data sets */

void getseqsoneset(FILE *ifp)
{      
  initidone(Maxspc);
  getseqsone(ifp);
}



/* print identifier in full 10 char length */
void fputidone10(FILE *ofp, int t, cmatrix Identif)
{	
  int i;
  
  for (i = 0; i < 10; i++) fputc(Identif[t][i], ofp);
}


/* print identifier of specified sequence up to first space */
int fputidone(FILE *ofp, int t, cmatrix Identif)
{	
  int i;
  
  i = 0;
  while (Identif[t][i] != ' ' && i < 10) {
    fputc(Identif[t][i], ofp);
    i++;
  }
  return i;
}


/* print sequences of specified sequence up to first space */
int fputaseq1(FILE *ofp, int t, int MaxseqL)
{	
  int i;
  
  i = 0;
  while (aseq1[t][i] != ' ' && i < MaxseqL) {
    fputc(aseq1[t][i], ofp);
    i++;
  }
  return i;
}



/*****************************************************************************/
/* open, close files                                                         */
/*****************************************************************************/



/* open input files */
void openinputfile(char *filename) {		

  /* open sequencefile  */
  aseq1fp = fopen(filename, "r");

  if (aseq1fp == NULL) {

    fprintf(stderr, "\nERROR: Missing sequence input file %s!\n", filename);
    exit (1);

  }
  
}


/* open output file */
void openoutputfile(char *filename) {

  oseqfp = fopen(filename, "w");

  if (oseqfp == NULL) {

    fprintf(stderr, "\nERROR: Could not open file %s for writing!\n",filename);
    exit(1);

  }
  
}	



/* close files */
void closefiles()
{	
  
  fclose(aseq1fp);
  fclose(oseqfp);
  
}



/*****************************************************************************/
/* routines                                                                  */
/*****************************************************************************/



/*****************************************************************************/
/* main part                                                                 */
/*****************************************************************************/

int main(int argc, char *argv[])
{       
  
  int spc, l, sizeofsusa, susaseqnr;
  int counter;
  double r;
  long seed;
  long *se;
  ivector Controletti;
  
  
  /* get sequences */
  if (argc != 6) {

    fprintf (stderr, "Usage: rasa infile outfile samplesize seqcount seqlength\n");
    exit (1);

  }

  openinputfile(argv [1]);
  
/*   fscanf(aseq1fp, "%d", &Maxspc); */
/*   fscanf(aseq1fp, "%d", &MaxseqL); */
  

  Maxspc = atoi (argv [4]);
  MaxseqL = atoi (argv [5]);

  getseqsoneset(aseq1fp);
  
  srand((unsigned)time(NULL));
  
  Controletti = new_ivector(Maxspc);
  for (l = 0; l < Maxspc; l++){
    Controletti[l] = 0;
  }
  
  /* open output files */
  openoutputfile(argv [2]);
    
  sizeofsusa = atoi (argv [3]);
   
  seed=-(long)rand();
  
  if(sizeofsusa >= Maxspc){

    fprintf(stderr,"sizeofSUBsample > sizeofsample !");

  }
  
  fprintf(oseqfp, "%d   %d \n", sizeofsusa, MaxseqL);           
  
  counter = 0;

  while(counter <= sizeofsusa){
    
    r = rand01(&seed);  
    susaseqnr = ( (int)(((float) Maxspc)*r) );
    
    if((Controletti[susaseqnr]) == 1){ 
      ;
    } else {
      Controletti[susaseqnr] = 1;   fprintf(stderr,"%d ", susaseqnr);
      
      fputidone10(oseqfp,susaseqnr,Idseq1);
      fputaseq1(oseqfp,susaseqnr,MaxseqL);
      fprintf(oseqfp, "\n");        
      
      counter++;
    }
    
  } /*while1**/
  
  /* garbage collection */
  free_cmatrix(Idseq1);
  free_cmatrix(aseq1);  
  
  /* close files */
  closefiles();

}

